*! version 7.0.6  28jan2002
program define corr2data
	version 7.0
	query sortseed
	local seed0 = r(sortseed)
	cap noi Make `0'
	set seed `seed0'
	if _rc { exit _rc }
end

program define Make 
	gettoken first 0: 0, parse(",")
	syntax [, n(string) CORR(string) COV(string) /*
		*/ Means(string) SDs(string) Double CLEAR]  

	qui count
	local curn = r(N)
	if "`n'" != "" {
		confirm integer n `n'	
		if `n' == `curn' { local n }
	}		
	if "`n'" == "" {	/* add newvarlist to existing dataset */
		local nobs = r(N)
		if `nobs' <= 0 { error 2000 }
		if "`clear'" != "" {
			drop _all 
			qui set obs `nobs' 
			local n `nobs'
		}	
	}
	else {			/* generate new dataset */	
		if `n' <= 0 { error 2000 }
		qui count
		if `n' != r(N) {
			qui des, short
			if r(changed) & ("`clear'" == "" ) { error 4 }
			drop _all
		}
		local nobs = `n'
		qui set obs `nobs' 
	}

	local 0 "`first'"
	syntax newvarlist 

	local k : word count `varlist'
	
	tempname P					
	mat `P' = I(`k')
							/* C = corr */
	if "`corr'" != "" {
		tempname C
		GetMat `C' corr `k' "`corr'" 
		local i = 1
		while `i' <= `k' {
			local j 1
			while `j' < = `k' {
				if `i' == `j' {
					cap assert `C'[`i',`j'] == 1
				}
				else { cap assert abs(`C'[`i',`j']) <= 1 }
				if _rc {
			di as err `"corr(`corr') incorrectly specified"'
			exit 198
				}
				local j = `j' + 1
			}
			local i = `i' + 1
		}
		mat `P' = `C'
		mat `P'= cholesky(`P')
	}
							/* V = cov */
	if "`cov'" != "" {
		if "`corr'" != "" { 
		dis as err "cov() and corr() may not be specified together"
		exit 198
		}
		tempname V S
		GetMat `V' cov `k' "`cov'"
		local i 1
		while `i' < = `k' {
			cap assert `V'[`i', `i'] >= 0
			if _rc {
			di as err `"cov(`cov') incorrectly specified"'
			exit 198
			}
			local i = `i' + 1
		}
		mat `S' = vecdiag(cholesky(`V'))
		mat `P' = `V'	
		mat `P'= cholesky(`P')
	}
							/* M = means */	
	tempname M
	if "`means'" != "" {
		GetMat `M' means `k' "`means'"
	}
	else { mat `M' = J(1,`k', 0) }
							/* S = stds */
	tempname S
	if "`sds'" != "" {
		if "`cov'" != "" {
		dis as err "cov() and sds() may not be specified together"
		exit 198
		}
		GetMat `S' sds `k' "`sds'"
		local i 1
		while `i' <= `k' {
			cap assert `S'[1,`i'] >= 0
			if _rc {
			di as err `"option sds(`sds') incorrectly specified"'
			exit 198
			}
			local i = `i' + 1
		}
	}
	else { mat `S' = J(1,`k',1) }

						/* generate new variables */
	set seed0 0
	foreach var of local varlist {
		qui gen `double' `var' = invnorm(uniform0())
		qui sum `var'
		qui replace `var' = `var' - r(mean)
	}						
					/* reform them to be zero corr */
	tempname T
	qui mat accum `T' = `varlist', noc dev
	mat `T' = `T'/(`nobs'-1)
	mat `T' = cholesky(syminv(`T'))
	local i 1
	while `i' < = `k' {
		tempname new`i' row
		mat `row' = (`T'[1..., `i'])'
		mat score `new`i'' = `row'
		local i = `i' + 1
	}
	local i 1
	tokenize `varlist'
	while `i' <= `k' {
		qui replace ``i'' = `new`i'' 
		local i = `i' + 1
	}
					/* transform to desired corr */
	mat roweq `P' = " "
	mat coleq `P' = " "	/* remove possible equation names from P */
	mat rownames `P' = `varlist'
	mat colnames `P' = `varlist'
	local i 1
	while `i' < = `k' {
		tempname new`i' row
		mat `row' = `P'[`i', 1...]
		mat score `new`i'' = `row'
		local i = `i' + 1
	}
	local i 1
	tokenize `varlist'
	while `i' <= `k' {
		qui replace ``i'' = `new`i'' 
		local i = `i' + 1
	}		

					/* transform to desired means and std */
	if "`cov'" != "" | "`sds'" != "" {
		local i 1
		while `i' <= `k' {
			qui replace ``i'' = ``i'' * `S'[1,`i']
			local i = `i' + 1
		}
	}
	if "`means'" != "" {
		local i 1
		tokenize `varlist'
		while `i' <= `k' {
			qui replace ``i'' = ``i'' + `M'[1,`i']
			local i = `i' +1
		}
	}
	if "`n'" != "" { dis as txt  "(obs `nobs')" }
end
	
program define GetMat  /* matname mattype #ofvarlist userdefine */
	args	T	/* matname 
	*/	type	/* type 
	*/	k	/* # of newvarlist 
	*/	A	/* user defined content */
	
 	
	cap mat list `A' 			/* matname? */
	if ! _rc {				/* yes */
		mat `T' = `A'
	}
	else {					/* no */
		cap mat input `T' = (`A')
		if _rc {
			dis as err "`type'(`A') invalid"
			exit 198
		}
	}
						/* right dimention? */
	if `k' != colsof(`T') {
		dis as err /*
	*/ "`k' variables specified but specified `type'(`A') is " /*
	*/ rowsof(`T') " x " colsof(`T')
		exit 503
	}
						/* symmetric ? */	
	if "`type'" == "corr" | "`type'" == "v" {
		if colsof(`T') != rowsof(`T') { error 505 }
	}
end  
